/*
 Copyright (C) 2024 3NSoft Inc.

 This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

import { GlobalEventSink } from "@/services";
import { ProcessStore } from "@/store/process/types";
import { defer } from "@v1nt1248/3nclient-lib/utils";

export async function installApp(
  appId: string, version: string, procsStore: ProcessStore,
  $emit?: GlobalEventSink
): Promise<void> {
  try {
    $emit?.('install:start', { appId, version });
    procsStore.upsertProcess(appId, {
      procType: 'installing'
    });
    await w3n.system!.apps!.installer!.installApp(appId, version);
    $emit?.('install:done', { appId, version });
  } catch  (err) {
    $emit?.('install:fail', { appId, version });
    throw err;
  } finally {
    procsStore.delProcess(appId);
  }
}

export async function downloadApp(
  appId: string, version: string, procsStore: ProcessStore,
  $emit?: GlobalEventSink
): Promise<void> {
  const deferred = defer<void>();
  procsStore.upsertProcess(appId, {
    procType: 'downloading',
    progressValue: 0
  });
  w3n.system!.apps!.downloader!.downloadWebApp(appId, version, {
    next: ev => {
      const { totalBytes, bytesLeft } = ev;
      const percents = Math.floor((totalBytes - bytesLeft)/totalBytes*100);
      procsStore.upsertProcess(appId, {
        procType: 'downloading',
        progressValue: percents
      });
    },
    complete: () => {
      deferred.resolve();
    },
    error: err => deferred.reject(err)
  });
  try {
    $emit?.('download:start', { appId, version });
    await deferred.promise;
    $emit?.('download:done', { appId, version });
  } catch  (err) {
    $emit?.('download:fail', { appId, version });
    throw err;
  } finally {
    procsStore.delProcess(appId);
  }
}
